package	jp.co.sds_corp.volcano;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.media.Format;
import javax.media.Manager;
import javax.media.Processor;
import javax.media.ProcessorModel;
import javax.media.format.AudioFormat;
import javax.media.format.VideoFormat;
import javax.media.protocol.FileTypeDescriptor;
import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;
import jp.co.sds_corp.util.window.StatusBar;
import jp.co.sds_corp.util.window.WindowUtility;

public class Sakurajima2 extends JFrame
{
	private static final long serialVersionUID = 1L;
	SetupDialog	setup;
	JMenuBar	menuBar;
	StatusBar	status;
	List<Image>	vecImg = new ArrayList<Image>();
	Map<String,Image>	mapImg = new HashMap<String,Image>();
	Settings	set = new Settings();
	boolean	bShow = false;

	public Sakurajima2() {
		super();
		setTitle( "桜島Viewer" );
		setSize( 650, 610 );
		createControl();
		setMenu();

		addWindowListener( new WindowAdapter() {
			public void windowClosing( WindowEvent e ) {
				closeWindow();
			}
		});
		setDefaultCloseOperation( WindowConstants.DO_NOTHING_ON_CLOSE );
	}

	private void closeWindow() {
		String msg = "終了しますか?";
		String title = "確認";
		int nRet = JOptionPane.showConfirmDialog( Sakurajima2.this,
					msg, title,
					JOptionPane.YES_NO_OPTION );
		if ( nRet == JOptionPane.YES_OPTION ) {
			System.exit( 0 );
		}
	}

	protected void createControl() {
		setup = new SetupDialog( this, set );
//		getContentPane().setLayout( new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ));

		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ));

		try {
			Format[] formats = new Format[2];
			formats[0] = new AudioFormat(AudioFormat.IMA4);
			formats[1] = new VideoFormat( VideoFormat.CINEPAK );
			FileTypeDescriptor outType =
					 new FileTypeDescriptor( FileTypeDescriptor.QUICKTIME );
			Processor proc = Manager.createRealizedProcessor(
				new ProcessorModel( formats, outType ));
			if ( proc.getVisualComponent() != null) {
				pnl.add( proc.getVisualComponent() );
				pnl.add( proc.getControlPanelComponent() );
			}
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
		status = new StatusBar();
		pnl.add( status );
		getContentPane().add( pnl, BorderLayout.SOUTH );
	}

	protected void setMenu() {
		menuBar = new JMenuBar();
		JMenu menu;
		JMenuItem item;

		menu = new JMenu( "ファイル(F)" );
		menu.setMnemonic( KeyEvent.VK_F );
		menuBar.add( menu );

		item = new JMenuItem( "設定(S)", KeyEvent.VK_S );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				setup();
			}
		});
		menu.add( item );
		item = new JMenuItem( "終了(X)", KeyEvent.VK_X );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				closeWindow();
			}
		});
		menu.add( item );

		menu = new JMenu( "取り込み(I)" );
		menu.setMnemonic( KeyEvent.VK_I );
		menuBar.add( menu );

		item = new JMenuItem( "開始(S)", KeyEvent.VK_S );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				readImage();
			}
		});
		menu.add( item );
		item = new JMenuItem( "中断(B)", KeyEvent.VK_B );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				stopRead();
			}
		});
		item.setEnabled( false );
		menu.add( item );

		menu = new JMenu( "ヘルプ(H)" );
		menu.setMnemonic( KeyEvent.VK_H );
		menuBar.add( menu );

		item = new JMenuItem( "桜島Viewerについて(A)", KeyEvent.VK_A );
		item.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				about();
			}
		});
		menu.add( item );

		setJMenuBar( menuBar );
	}

	Thread readThr;
	Thread aniThr;
	protected void readImage() {
		if ( readThr == null ) {
			readThr = new Thread( new Reader());
			readThr.start();
		}
	}
	protected void stopRead() {
		if ( readThr != null ) {
			readThr.interrupt();
		}
	}
	private void readEnabled( boolean en ) {
		menuBar.getMenu( 1 ).getItem( 0 ).setEnabled( !en );
		menuBar.getMenu( 1 ).getItem( 1 ).setEnabled( en );
	}

	protected void startAnimation() {
		if ( aniThr == null ) {
			aniThr = new Thread( new Animation());
			aniThr.start();
		}
	}
	protected void stopAnimation() {
		if ( aniThr != null ) {
			aniThr.interrupt();
		}
	}
	protected void setup() {
		WindowUtility.center( this, setup );
		setup.setVisible(true);
	}

	protected void showImage( int idx ) {
		if ( vecImg.size() > idx ) {
			BufferedImage img = (BufferedImage )vecImg.get( idx );
			bShow = true;
		}
	}

	protected void about() {
	}

	class Reader implements Runnable
	{
		public void run() {
			readEnabled( true );
			try {
				ISource src = set.getSource();
				src.setUrl( new URL( set.getUrl() ));
				vecImg.clear();
				int num = 0;
				List<URL> list = src.listURL();
				for ( int i = 0; i < list.size(); i++ ) {
					if ( Thread.currentThread().isInterrupted()) {
						break;
					}
					URL file = (URL )list.get( i );
					BufferedImage img;
					img = (BufferedImage )mapImg.get( file.toString() );
					if ( img != null ) {
						vecImg.add( img );
						num++;
						continue;
					}
					img = ImageIO.read( file );
					if ( img != null ) {
						vecImg.add( img );
						mapImg.put( file.toString(), img );
						status.setText( 0, file.toString() );
						if ( !bShow ) {
							showImage( num );
						}
						num++;
					}
				}
			} catch ( Exception ex ) {
				ex.printStackTrace();
			}
			readThr = null;
			readEnabled( false );
		}
	}

	class Animation implements Runnable
	{
		public void run() {
			try {
				for ( ;; ) {
					if ( Thread.currentThread().isInterrupted()) {
						break;
					}
					int pos = set.getPosition();
					showImage( pos );
					if ( set.getDirection() > 0 ) {
						if (( pos + 1 ) >= vecImg.size()) {
							break;
						}
					} else if ( set.getDirection() < 0 ) {
						if ( pos == 0 ) {
							break;
						}
					}
					pos += set.getDirection();
					set.setPosition( pos );
					try {
						Thread.sleep( set.getInterval() );
					} catch ( InterruptedException iex ) {
						break;
					}
				}
			} catch ( Exception ex ) {
				ex.printStackTrace();
			}
			aniThr = null;
		}
	}

	public static void main( String argv[] ) {
		CmnLogger logger = CmnLogFactory.getInstance().getLogger(Sakurajima2.class);
		try {
			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
		} catch (Exception exc) {
			System.err.println( "Error loading L&F: " + exc);
		}
		try {
			Sakurajima2 win = new Sakurajima2();
			// win.init( config );
			win.setVisible(true);
			win.toFront();
		} catch ( Throwable ex ) {
			logger.error( "initialize error", ex );
			JOptionPane.showMessageDialog( null,
				"Error occurred.\n" + ex,
				"Error",
				JOptionPane.ERROR_MESSAGE );
			System.exit( 9 );
		}
	}

}
